package com.huimv.video.service.impl;

import com.baomidou.dynamic.datasource.annotation.DS;
import com.huimv.video.domain.MobileUser;
import com.huimv.video.repo.MobileUserRepository;
import com.huimv.video.result.Result;
import com.huimv.video.result.ResultStatus;
import com.huimv.video.service.MobileUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.RequestMapping;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class MobileUserServiceImpl implements MobileUserService {

	@Autowired
	private MobileUserRepository rep;

	@Override
	@DS("master")
	public Result add(MobileUser entity){
		if (entity == null){
			return new Result(10002, ResultStatus.addNull);
		}
		try {
			rep.save(entity);
			return new Result(10000,ResultStatus.addSuccess);
		}catch (Exception e){
			return new Result(10001,ResultStatus.addFailed);
		}
	}

	@Override
	@DS("master")
	public Result remove(Integer[] ids) {
		if (ids == null || ids.length==0){
			return new Result(10002,ResultStatus.deleteNull);
		}
		try {
			for (Integer id : ids) {
				rep.deleteById(id);
			}
			return new Result(10000,ResultStatus.deleteSuccess);
		}catch (Exception e){
			return new Result(10001,ResultStatus.deleteFailed);
		}
	}

	@Override
	@DS("master")
	public Result update(MobileUser entity) {
		if (entity == null){
			return new Result(10002,ResultStatus.updateNull);
		}
		try {
			rep.save(entity);
			return new Result(10000,ResultStatus.updateSuccess);
		}catch (Exception e){
			return new Result(10001,ResultStatus.updateFailed);
		}
	}

	@Override
	@DS("master")
	public Result findAll(String name , Integer pageNum , Integer pageSize) {
		try {
			Map map = new HashMap();
			int size = rep.findAll().size() ;
			Integer startPage = (pageNum-1) * pageSize;
			List<MobileUser> all = rep.findAll(name,startPage,pageSize);
			map.put("total",size);
			map.put("totalPageNum",(size  +  pageSize  - 1) / pageSize);
			map.put("data",all);
			
			return new Result(10000,ResultStatus.findSuccess,map);
		}catch (Exception e){
			return new Result(10001,ResultStatus.findFailed,null);
		}
	}

	@Override
	@DS("master")
	public Result findAllById(Integer id) {
		try {
			MobileUser entity = rep.findById(id).get();
			return new Result(10000,ResultStatus.findSuccess,entity);
		}catch (Exception e){
			return new Result(10001,ResultStatus.findFailed,null);
		}
	}

	@Override
	@DS("master")
	public Result login(String account, String password) {
		MobileUser byUserNameAndPassword = rep.findByAccountAndPassword(account, password);
		if (byUserNameAndPassword == null){
			return new Result(10001,"登录失败",null);
		}
		return new Result(10000,"登录成功",byUserNameAndPassword);
	}
}
